/*
	EmacsClientRead.cpp  ---  Jonathan Arnold   December 10 1997

	Class to reads GNUSERV's client mailslot

	This file is not part of GNU Emacs. 

	Copyright(C) 1999 Jonathan Arnold (jdarnold@buddydog.org)

	A TThread object that reads the mailslot
	  \\.\mailslot\emacs\gnuclient\CLIENTID
	where CLIENTID is the 8 hex character process id used by GNUSERV to
	communicate back to us.  It basically just passes the info on back to
	our parent EmacsEdit object.

	*/

//---------------------------------------------------------------------------
#include <vcl\classes.hpp>
#pragma hdrstop

#include "gnuserv.h"

#include "EmacsClientRead.h"

__fastcall EmacsClientRead::EmacsClientRead(TEmacsEdit* Parent,
																						bool CreateSuspended)
  : TThread(CreateSuspended), parent(Parent) 
{
//	OutputDebugString( "EmacsClientRead constructor 1\n" );

	FreeOnTerminate = true;
}

void EmacsClientRead::SetMailslot( HANDLE slot )
{
	resultSlot = slot;
}

bool EmacsClientRead::IsValid()
{
	return INVALID_HANDLE_VALUE != resultSlot;
}

//---------------------------------------------------------------------------
void __fastcall EmacsClientRead::Execute()
{
	Priority = tpIdle;

	while ( !Terminated )
	{
		char result[2000];
		DWORD bytesread;
		DWORD werrno;
		
		bytesread = 0;
		bool okay = ReadFile (resultSlot, result, sizeof (result), &bytesread, NULL);

		if ( !okay && (werrno = GetLastError()) != ERROR_SEM_TIMEOUT )
		{ // show error
			char errstr[100];
			FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM, NULL, werrno, 0,
										 errstr, sizeof(errstr), NULL );
			MessageBox( NULL, errstr, "GNUClient ReadError", MB_OK );
			break;
		}
		else if (bytesread)
		{

#if 1
			OutputDebugString( "result = " );
			OutputDebugString( result );
			OutputDebugString( "\n" );

			OutputDebugString( "last chars = " );

			{
				char buff[111];
				sprintf(buff,"0x%x 0x%x 0x%x 0x%x\n",
								*(result+(strlen(result)-4)),
								*(result+(strlen(result)-3)),
								*(result+(strlen(result)-2)),
								*(result+(strlen(result)-1)) );
				OutputDebugString( buff );
			}
#endif

			for ( int ix=strlen(result)-4; result[ix]; ix++ )
			{ // find first non-printable char
				if ( iscntrl( result[ix] ) )
				{ //found it, so end string here
					result[ix] = 0;
					break;
				}
			}
			
			msg = new char[strlen(result)+1];
			strcpy( msg, result );

			Synchronize( SendMsgBack );
		}

		if ( !Terminated )
		{
			Sleep(500);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall EmacsClientRead::SendMsgBack(void)
{
	parent->MsgFromEmacs( msg );
	delete msg;
	msg=NULL;
}

__fastcall EmacsClientRead::~EmacsClientRead()
{

}
	
	
